@@ -16,16 +16,21 @@
 from collections import OrderedDict
 import functools
 import re
-from typing import Dict, Sequence, Tuple, Type, Union
+from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-import google.api_core.client_options as ClientOptions  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core.client_options import ClientOptions
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
+
 from google.cloud.appengine_admin_v1.services.authorized_certificates import pagers
 from google.cloud.appengine_admin_v1.types import appengine
 from google.cloud.appengine_admin_v1.types import certificate
@@ -106,6 +111,42 @@ class AuthorizedCertificatesAsyncClient:
 
     from_service_account_json = from_service_account_file
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        return AuthorizedCertificatesClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore
+
     @property
     def transport(self) -> AuthorizedCertificatesTransport:
         """Returns the transport used by the client instance.
@@ -169,9 +210,9 @@ class AuthorizedCertificatesAsyncClient:
 
     async def list_authorized_certificates(
         self,
-        request: appengine.ListAuthorizedCertificatesRequest = None,
+        request: Union[appengine.ListAuthorizedCertificatesRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> pagers.ListAuthorizedCertificatesAsyncPager:
@@ -179,7 +220,7 @@ class AuthorizedCertificatesAsyncClient:
         administer.
 
         Args:
-            request (:class:`google.cloud.appengine_admin_v1.types.ListAuthorizedCertificatesRequest`):
+            request (Union[google.cloud.appengine_admin_v1.types.ListAuthorizedCertificatesRequest, dict]):
                 The request object. Request message for
                 `AuthorizedCertificates.ListAuthorizedCertificates`.
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -228,16 +269,16 @@ class AuthorizedCertificatesAsyncClient:
 
     async def get_authorized_certificate(
         self,
-        request: appengine.GetAuthorizedCertificateRequest = None,
+        request: Union[appengine.GetAuthorizedCertificateRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> certificate.AuthorizedCertificate:
         r"""Gets the specified SSL certificate.
 
         Args:
-            request (:class:`google.cloud.appengine_admin_v1.types.GetAuthorizedCertificateRequest`):
+            request (Union[google.cloud.appengine_admin_v1.types.GetAuthorizedCertificateRequest, dict]):
                 The request object. Request message for
                 `AuthorizedCertificates.GetAuthorizedCertificate`.
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -280,16 +321,16 @@ class AuthorizedCertificatesAsyncClient:
 
     async def create_authorized_certificate(
         self,
-        request: appengine.CreateAuthorizedCertificateRequest = None,
+        request: Union[appengine.CreateAuthorizedCertificateRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> certificate.AuthorizedCertificate:
         r"""Uploads the specified SSL certificate.
 
         Args:
-            request (:class:`google.cloud.appengine_admin_v1.types.CreateAuthorizedCertificateRequest`):
+            request (Union[google.cloud.appengine_admin_v1.types.CreateAuthorizedCertificateRequest, dict]):
                 The request object. Request message for
                 `AuthorizedCertificates.CreateAuthorizedCertificate`.
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -332,9 +373,9 @@ class AuthorizedCertificatesAsyncClient:
 
     async def update_authorized_certificate(
         self,
-        request: appengine.UpdateAuthorizedCertificateRequest = None,
+        request: Union[appengine.UpdateAuthorizedCertificateRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> certificate.AuthorizedCertificate:
@@ -346,7 +387,7 @@ class AuthorizedCertificatesAsyncClient:
         updated.
 
         Args:
-            request (:class:`google.cloud.appengine_admin_v1.types.UpdateAuthorizedCertificateRequest`):
+            request (Union[google.cloud.appengine_admin_v1.types.UpdateAuthorizedCertificateRequest, dict]):
                 The request object. Request message for
                 `AuthorizedCertificates.UpdateAuthorizedCertificate`.
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -389,16 +430,16 @@ class AuthorizedCertificatesAsyncClient:
 
     async def delete_authorized_certificate(
         self,
-        request: appengine.DeleteAuthorizedCertificateRequest = None,
+        request: Union[appengine.DeleteAuthorizedCertificateRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> None:
         r"""Deletes the specified SSL certificate.
 
         Args:
-            request (:class:`google.cloud.appengine_admin_v1.types.DeleteAuthorizedCertificateRequest`):
+            request (Union[google.cloud.appengine_admin_v1.types.DeleteAuthorizedCertificateRequest, dict]):
                 The request object. Request message for
                 `AuthorizedCertificates.DeleteAuthorizedCertificate`.
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -429,6 +470,12 @@ class AuthorizedCertificatesAsyncClient:
             request, retry=retry, timeout=timeout, metadata=metadata,
         )
 
+    async def __aenter__(self):
+        return self
+
+    async def __aexit__(self, exc_type, exc, tb):
+        await self.transport.close()
+
 
 try:
     DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
