@@ -16,11 +16,10 @@
 import warnings
 from typing import Awaitable, Callable, Dict, Optional, Sequence, Tuple, Union
 
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import grpc_helpers_async  # type: ignore
+from google.api_core import gapic_v1
+from google.api_core import grpc_helpers_async
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.auth.transport.grpc import SslCredentials  # type: ignore
-import packaging.version
 
 import grpc  # type: ignore
 from grpc.experimental import aio  # type: ignore
@@ -388,5 +387,8 @@ class AuthorizedCertificatesGrpcAsyncIOTransport(AuthorizedCertificatesTransport
             )
         return self._stubs["delete_authorized_certificate"]
 
+    def close(self):
+        return self.grpc_channel.close()
+
 
 __all__ = ("AuthorizedCertificatesGrpcAsyncIOTransport",)
