@@ -16,16 +16,21 @@
 from collections import OrderedDict
 import functools
 import re
-from typing import Dict, Sequence, Tuple, Type, Union
+from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-import google.api_core.client_options as ClientOptions  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core.client_options import ClientOptions
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
+
 from google.cloud.appengine_admin_v1.services.firewall import pagers
 from google.cloud.appengine_admin_v1.types import appengine
 from google.cloud.appengine_admin_v1.types import firewall
@@ -103,6 +108,42 @@ class FirewallAsyncClient:
 
     from_service_account_json = from_service_account_file
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        return FirewallClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore
+
     @property
     def transport(self) -> FirewallTransport:
         """Returns the transport used by the client instance.
@@ -165,16 +206,16 @@ class FirewallAsyncClient:
 
     async def list_ingress_rules(
         self,
-        request: appengine.ListIngressRulesRequest = None,
+        request: Union[appengine.ListIngressRulesRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> pagers.ListIngressRulesAsyncPager:
         r"""Lists the firewall rules of an application.
 
         Args:
-            request (:class:`google.cloud.appengine_admin_v1.types.ListIngressRulesRequest`):
+            request (Union[google.cloud.appengine_admin_v1.types.ListIngressRulesRequest, dict]):
                 The request object. Request message for
                 `Firewall.ListIngressRules`.
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -222,9 +263,9 @@ class FirewallAsyncClient:
 
     async def batch_update_ingress_rules(
         self,
-        request: appengine.BatchUpdateIngressRulesRequest = None,
+        request: Union[appengine.BatchUpdateIngressRulesRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> appengine.BatchUpdateIngressRulesResponse:
@@ -237,7 +278,7 @@ class FirewallAsyncClient:
         end of the list.
 
         Args:
-            request (:class:`google.cloud.appengine_admin_v1.types.BatchUpdateIngressRulesRequest`):
+            request (Union[google.cloud.appengine_admin_v1.types.BatchUpdateIngressRulesRequest, dict]):
                 The request object. Request message for
                 `Firewall.BatchUpdateIngressRules`.
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -275,16 +316,16 @@ class FirewallAsyncClient:
 
     async def create_ingress_rule(
         self,
-        request: appengine.CreateIngressRuleRequest = None,
+        request: Union[appengine.CreateIngressRuleRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> firewall.FirewallRule:
         r"""Creates a firewall rule for the application.
 
         Args:
-            request (:class:`google.cloud.appengine_admin_v1.types.CreateIngressRuleRequest`):
+            request (Union[google.cloud.appengine_admin_v1.types.CreateIngressRuleRequest, dict]):
                 The request object. Request message for
                 `Firewall.CreateIngressRule`.
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -326,16 +367,16 @@ class FirewallAsyncClient:
 
     async def get_ingress_rule(
         self,
-        request: appengine.GetIngressRuleRequest = None,
+        request: Union[appengine.GetIngressRuleRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> firewall.FirewallRule:
         r"""Gets the specified firewall rule.
 
         Args:
-            request (:class:`google.cloud.appengine_admin_v1.types.GetIngressRuleRequest`):
+            request (Union[google.cloud.appengine_admin_v1.types.GetIngressRuleRequest, dict]):
                 The request object. Request message for
                 `Firewall.GetIngressRule`.
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -377,16 +418,16 @@ class FirewallAsyncClient:
 
     async def update_ingress_rule(
         self,
-        request: appengine.UpdateIngressRuleRequest = None,
+        request: Union[appengine.UpdateIngressRuleRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> firewall.FirewallRule:
         r"""Updates the specified firewall rule.
 
         Args:
-            request (:class:`google.cloud.appengine_admin_v1.types.UpdateIngressRuleRequest`):
+            request (Union[google.cloud.appengine_admin_v1.types.UpdateIngressRuleRequest, dict]):
                 The request object. Request message for
                 `Firewall.UpdateIngressRule`.
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -428,16 +469,16 @@ class FirewallAsyncClient:
 
     async def delete_ingress_rule(
         self,
-        request: appengine.DeleteIngressRuleRequest = None,
+        request: Union[appengine.DeleteIngressRuleRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> None:
         r"""Deletes the specified firewall rule.
 
         Args:
-            request (:class:`google.cloud.appengine_admin_v1.types.DeleteIngressRuleRequest`):
+            request (Union[google.cloud.appengine_admin_v1.types.DeleteIngressRuleRequest, dict]):
                 The request object. Request message for
                 `Firewall.DeleteIngressRule`.
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -468,6 +509,12 @@ class FirewallAsyncClient:
             request, retry=retry, timeout=timeout, metadata=metadata,
         )
 
+    async def __aenter__(self):
+        return self
+
+    async def __aexit__(self, exc_type, exc, tb):
+        await self.transport.close()
+
 
 try:
     DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
