@@ -16,16 +16,21 @@
 from collections import OrderedDict
 import functools
 import re
-from typing import Dict, Sequence, Tuple, Type, Union
+from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-import google.api_core.client_options as ClientOptions  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core.client_options import ClientOptions
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
+
 from google.api_core import operation as gac_operation  # type: ignore
 from google.api_core import operation_async  # type: ignore
 from google.cloud.appengine_admin_v1.services.versions import pagers
@@ -100,6 +105,42 @@ class VersionsAsyncClient:
 
     from_service_account_json = from_service_account_file
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        return VersionsClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore
+
     @property
     def transport(self) -> VersionsTransport:
         """Returns the transport used by the client instance.
@@ -162,16 +203,16 @@ class VersionsAsyncClient:
 
     async def list_versions(
         self,
-        request: appengine.ListVersionsRequest = None,
+        request: Union[appengine.ListVersionsRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> pagers.ListVersionsAsyncPager:
         r"""Lists the versions of a service.
 
         Args:
-            request (:class:`google.cloud.appengine_admin_v1.types.ListVersionsRequest`):
+            request (Union[google.cloud.appengine_admin_v1.types.ListVersionsRequest, dict]):
                 The request object. Request message for
                 `Versions.ListVersions`.
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -219,9 +260,9 @@ class VersionsAsyncClient:
 
     async def get_version(
         self,
-        request: appengine.GetVersionRequest = None,
+        request: Union[appengine.GetVersionRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> version.Version:
@@ -230,7 +271,7 @@ class VersionsAsyncClient:
         parameter to get the full resource.
 
         Args:
-            request (:class:`google.cloud.appengine_admin_v1.types.GetVersionRequest`):
+            request (Union[google.cloud.appengine_admin_v1.types.GetVersionRequest, dict]):
                 The request object. Request message for
                 `Versions.GetVersion`.
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -271,16 +312,16 @@ class VersionsAsyncClient:
 
     async def create_version(
         self,
-        request: appengine.CreateVersionRequest = None,
+        request: Union[appengine.CreateVersionRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation_async.AsyncOperation:
         r"""Deploys code and resource files to a new version.
 
         Args:
-            request (:class:`google.cloud.appengine_admin_v1.types.CreateVersionRequest`):
+            request (Union[google.cloud.appengine_admin_v1.types.CreateVersionRequest, dict]):
                 The request object. Request message for
                 `Versions.CreateVersion`.
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -330,9 +371,9 @@ class VersionsAsyncClient:
 
     async def update_version(
         self,
-        request: appengine.UpdateVersionRequest = None,
+        request: Union[appengine.UpdateVersionRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation_async.AsyncOperation:
@@ -374,7 +415,7 @@ class VersionsAsyncClient:
         -  ```manual_scaling.instances`` <https://cloud.google.com/appengine/docs/admin-api/reference/rest/v1/apps.services.versions#manualscaling>`__
 
         Args:
-            request (:class:`google.cloud.appengine_admin_v1.types.UpdateVersionRequest`):
+            request (Union[google.cloud.appengine_admin_v1.types.UpdateVersionRequest, dict]):
                 The request object. Request message for
                 `Versions.UpdateVersion`.
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -424,16 +465,16 @@ class VersionsAsyncClient:
 
     async def delete_version(
         self,
-        request: appengine.DeleteVersionRequest = None,
+        request: Union[appengine.DeleteVersionRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation_async.AsyncOperation:
         r"""Deletes an existing Version resource.
 
         Args:
-            request (:class:`google.cloud.appengine_admin_v1.types.DeleteVersionRequest`):
+            request (Union[google.cloud.appengine_admin_v1.types.DeleteVersionRequest, dict]):
                 The request object. Request message for
                 `Versions.DeleteVersion`.
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -492,6 +533,12 @@ class VersionsAsyncClient:
         # Done; return the response.
         return response
 
+    async def __aenter__(self):
+        return self
+
+    async def __aexit__(self, exc_type, exc, tb):
+        await self.transport.close()
+
 
 try:
     DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
