@@ -26,13 +26,25 @@ __protobuf__ = proto.module(
 
 class AuditData(proto.Message):
     r"""App Engine admin service audit log.
+
+    This message has `oneof`_ fields (mutually exclusive fields).
+    For each oneof, at most one member field can be set at the same time.
+    Setting any member of the oneof automatically clears all other
+    members.
+
+    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields
+
     Attributes:
         update_service (google.cloud.appengine_admin_v1.types.UpdateServiceMethod):
             Detailed information about UpdateService
             call.
+
+            This field is a member of `oneof`_ ``method``.
         create_version (google.cloud.appengine_admin_v1.types.CreateVersionMethod):
             Detailed information about CreateVersion
             call.
+
+            This field is a member of `oneof`_ ``method``.
     """
 
     update_service = proto.Field(
@@ -45,6 +57,7 @@ class AuditData(proto.Message):
 
 class UpdateServiceMethod(proto.Message):
     r"""Detailed information about UpdateService call.
+
     Attributes:
         request (google.cloud.appengine_admin_v1.types.UpdateServiceRequest):
             Update service request.
@@ -57,6 +70,7 @@ class UpdateServiceMethod(proto.Message):
 
 class CreateVersionMethod(proto.Message):
     r"""Detailed information about CreateVersion call.
+
     Attributes:
         request (google.cloud.appengine_admin_v1.types.CreateVersionRequest):
             Create version request.
