@@ -15,7 +15,6 @@
 #
 import os
 import mock
-import packaging.version
 
 import grpc
 from grpc.experimental import aio
@@ -29,15 +28,13 @@ from google.api_core import exceptions as core_exceptions
 from google.api_core import gapic_v1
 from google.api_core import grpc_helpers
 from google.api_core import grpc_helpers_async
+from google.api_core import path_template
 from google.auth import credentials as ga_credentials
 from google.auth.exceptions import MutualTLSChannelError
 from google.cloud.appengine_admin_v1.services.firewall import FirewallAsyncClient
 from google.cloud.appengine_admin_v1.services.firewall import FirewallClient
 from google.cloud.appengine_admin_v1.services.firewall import pagers
 from google.cloud.appengine_admin_v1.services.firewall import transports
-from google.cloud.appengine_admin_v1.services.firewall.transports.base import (
-    _GOOGLE_AUTH_VERSION,
-)
 from google.cloud.appengine_admin_v1.types import appengine
 from google.cloud.appengine_admin_v1.types import firewall
 from google.oauth2 import service_account
@@ -45,20 +42,6 @@ from google.protobuf import field_mask_pb2  # type: ignore
 import google.auth
 
 
-# TODO(busunkim): Once google-auth >= 1.25.0 is required transitively
-# through google-api-core:
-# - Delete the auth "less than" test cases
-# - Delete these pytest markers (Make the "greater than or equal to" tests the default).
-requires_google_auth_lt_1_25_0 = pytest.mark.skipif(
-    packaging.version.parse(_GOOGLE_AUTH_VERSION) >= packaging.version.parse("1.25.0"),
-    reason="This test requires google-auth < 1.25.0",
-)
-requires_google_auth_gte_1_25_0 = pytest.mark.skipif(
-    packaging.version.parse(_GOOGLE_AUTH_VERSION) < packaging.version.parse("1.25.0"),
-    reason="This test requires google-auth >= 1.25.0",
-)
-
-
 def client_cert_source_callback():
     return b"cert bytes", b"key bytes"
 
@@ -198,7 +181,7 @@ def test_firewall_client_client_options(client_class, transport_class, transport
     options = client_options.ClientOptions(api_endpoint="squid.clam.whelk")
     with mock.patch.object(transport_class, "__init__") as patched:
         patched.return_value = None
-        client = client_class(client_options=options)
+        client = client_class(transport=transport_name, client_options=options)
         patched.assert_called_once_with(
             credentials=None,
             credentials_file=None,
@@ -215,7 +198,7 @@ def test_firewall_client_client_options(client_class, transport_class, transport
     with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "never"}):
         with mock.patch.object(transport_class, "__init__") as patched:
             patched.return_value = None
-            client = client_class()
+            client = client_class(transport=transport_name)
             patched.assert_called_once_with(
                 credentials=None,
                 credentials_file=None,
@@ -232,7 +215,7 @@ def test_firewall_client_client_options(client_class, transport_class, transport
     with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "always"}):
         with mock.patch.object(transport_class, "__init__") as patched:
             patched.return_value = None
-            client = client_class()
+            client = client_class(transport=transport_name)
             patched.assert_called_once_with(
                 credentials=None,
                 credentials_file=None,
@@ -248,20 +231,20 @@ def test_firewall_client_client_options(client_class, transport_class, transport
     # unsupported value.
     with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "Unsupported"}):
         with pytest.raises(MutualTLSChannelError):
-            client = client_class()
+            client = client_class(transport=transport_name)
 
     # Check the case GOOGLE_API_USE_CLIENT_CERTIFICATE has unsupported value.
     with mock.patch.dict(
         os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "Unsupported"}
     ):
         with pytest.raises(ValueError):
-            client = client_class()
+            client = client_class(transport=transport_name)
 
     # Check the case quota_project_id is provided
     options = client_options.ClientOptions(quota_project_id="octopus")
     with mock.patch.object(transport_class, "__init__") as patched:
         patched.return_value = None
-        client = client_class(client_options=options)
+        client = client_class(client_options=options, transport=transport_name)
         patched.assert_called_once_with(
             credentials=None,
             credentials_file=None,
@@ -318,7 +301,7 @@ def test_firewall_client_mtls_env_auto(
         )
         with mock.patch.object(transport_class, "__init__") as patched:
             patched.return_value = None
-            client = client_class(client_options=options)
+            client = client_class(client_options=options, transport=transport_name)
 
             if use_client_cert_env == "false":
                 expected_client_cert_source = None
@@ -360,7 +343,7 @@ def test_firewall_client_mtls_env_auto(
                         expected_client_cert_source = client_cert_source_callback
 
                     patched.return_value = None
-                    client = client_class()
+                    client = client_class(transport=transport_name)
                     patched.assert_called_once_with(
                         credentials=None,
                         credentials_file=None,
@@ -382,7 +365,7 @@ def test_firewall_client_mtls_env_auto(
                 return_value=False,
             ):
                 patched.return_value = None
-                client = client_class()
+                client = client_class(transport=transport_name)
                 patched.assert_called_once_with(
                     credentials=None,
                     credentials_file=None,
@@ -395,6 +378,83 @@ def test_firewall_client_mtls_env_auto(
                 )
 
 
+@pytest.mark.parametrize("client_class", [FirewallClient, FirewallAsyncClient])
+@mock.patch.object(
+    FirewallClient, "DEFAULT_ENDPOINT", modify_default_endpoint(FirewallClient)
+)
+@mock.patch.object(
+    FirewallAsyncClient,
+    "DEFAULT_ENDPOINT",
+    modify_default_endpoint(FirewallAsyncClient),
+)
+def test_firewall_client_get_mtls_endpoint_and_cert_source(client_class):
+    mock_client_cert_source = mock.Mock()
+
+    # Test the case GOOGLE_API_USE_CLIENT_CERTIFICATE is "true".
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "true"}):
+        mock_api_endpoint = "foo"
+        options = client_options.ClientOptions(
+            client_cert_source=mock_client_cert_source, api_endpoint=mock_api_endpoint
+        )
+        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source(
+            options
+        )
+        assert api_endpoint == mock_api_endpoint
+        assert cert_source == mock_client_cert_source
+
+    # Test the case GOOGLE_API_USE_CLIENT_CERTIFICATE is "false".
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "false"}):
+        mock_client_cert_source = mock.Mock()
+        mock_api_endpoint = "foo"
+        options = client_options.ClientOptions(
+            client_cert_source=mock_client_cert_source, api_endpoint=mock_api_endpoint
+        )
+        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source(
+            options
+        )
+        assert api_endpoint == mock_api_endpoint
+        assert cert_source is None
+
+    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "never".
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "never"}):
+        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source()
+        assert api_endpoint == client_class.DEFAULT_ENDPOINT
+        assert cert_source is None
+
+    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "always".
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "always"}):
+        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source()
+        assert api_endpoint == client_class.DEFAULT_MTLS_ENDPOINT
+        assert cert_source is None
+
+    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "auto" and default cert doesn't exist.
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "true"}):
+        with mock.patch(
+            "google.auth.transport.mtls.has_default_client_cert_source",
+            return_value=False,
+        ):
+            api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source()
+            assert api_endpoint == client_class.DEFAULT_ENDPOINT
+            assert cert_source is None
+
+    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "auto" and default cert exists.
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "true"}):
+        with mock.patch(
+            "google.auth.transport.mtls.has_default_client_cert_source",
+            return_value=True,
+        ):
+            with mock.patch(
+                "google.auth.transport.mtls.default_client_cert_source",
+                return_value=mock_client_cert_source,
+            ):
+                (
+                    api_endpoint,
+                    cert_source,
+                ) = client_class.get_mtls_endpoint_and_cert_source()
+                assert api_endpoint == client_class.DEFAULT_MTLS_ENDPOINT
+                assert cert_source == mock_client_cert_source
+
+
 @pytest.mark.parametrize(
     "client_class,transport_class,transport_name",
     [
@@ -409,7 +469,7 @@ def test_firewall_client_client_options_scopes(
     options = client_options.ClientOptions(scopes=["1", "2"],)
     with mock.patch.object(transport_class, "__init__") as patched:
         patched.return_value = None
-        client = client_class(client_options=options)
+        client = client_class(client_options=options, transport=transport_name)
         patched.assert_called_once_with(
             credentials=None,
             credentials_file=None,
@@ -436,7 +496,7 @@ def test_firewall_client_client_options_credentials_file(
     options = client_options.ClientOptions(credentials_file="credentials.json")
     with mock.patch.object(transport_class, "__init__") as patched:
         patched.return_value = None
-        client = client_class(client_options=options)
+        client = client_class(client_options=options, transport=transport_name)
         patched.assert_called_once_with(
             credentials=None,
             credentials_file="credentials.json",
@@ -467,9 +527,8 @@ def test_firewall_client_client_options_from_dict():
         )
 
 
-def test_list_ingress_rules(
-    transport: str = "grpc", request_type=appengine.ListIngressRulesRequest
-):
+@pytest.mark.parametrize("request_type", [appengine.ListIngressRulesRequest, dict,])
+def test_list_ingress_rules(request_type, transport: str = "grpc"):
     client = FirewallClient(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -498,10 +557,6 @@ def test_list_ingress_rules(
     assert response.next_page_token == "next_page_token_value"
 
 
-def test_list_ingress_rules_from_dict():
-    test_list_ingress_rules(request_type=dict)
-
-
 def test_list_ingress_rules_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -611,8 +666,10 @@ async def test_list_ingress_rules_field_headers_async():
     assert ("x-goog-request-params", "parent=parent/value",) in kw["metadata"]
 
 
-def test_list_ingress_rules_pager():
-    client = FirewallClient(credentials=ga_credentials.AnonymousCredentials,)
+def test_list_ingress_rules_pager(transport_name: str = "grpc"):
+    client = FirewallClient(
+        credentials=ga_credentials.AnonymousCredentials, transport=transport_name,
+    )
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(
@@ -653,8 +710,10 @@ def test_list_ingress_rules_pager():
         assert all(isinstance(i, firewall.FirewallRule) for i in results)
 
 
-def test_list_ingress_rules_pages():
-    client = FirewallClient(credentials=ga_credentials.AnonymousCredentials,)
+def test_list_ingress_rules_pages(transport_name: str = "grpc"):
+    client = FirewallClient(
+        credentials=ga_credentials.AnonymousCredentials, transport=transport_name,
+    )
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(
@@ -765,9 +824,10 @@ async def test_list_ingress_rules_async_pages():
             assert page_.raw_page.next_page_token == token
 
 
-def test_batch_update_ingress_rules(
-    transport: str = "grpc", request_type=appengine.BatchUpdateIngressRulesRequest
-):
+@pytest.mark.parametrize(
+    "request_type", [appengine.BatchUpdateIngressRulesRequest, dict,]
+)
+def test_batch_update_ingress_rules(request_type, transport: str = "grpc"):
     client = FirewallClient(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -793,10 +853,6 @@ def test_batch_update_ingress_rules(
     assert isinstance(response, appengine.BatchUpdateIngressRulesResponse)
 
 
-def test_batch_update_ingress_rules_from_dict():
-    test_batch_update_ingress_rules(request_type=dict)
-
-
 def test_batch_update_ingress_rules_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -906,9 +962,8 @@ async def test_batch_update_ingress_rules_field_headers_async():
     assert ("x-goog-request-params", "name=name/value",) in kw["metadata"]
 
 
-def test_create_ingress_rule(
-    transport: str = "grpc", request_type=appengine.CreateIngressRuleRequest
-):
+@pytest.mark.parametrize("request_type", [appengine.CreateIngressRuleRequest, dict,])
+def test_create_ingress_rule(request_type, transport: str = "grpc"):
     client = FirewallClient(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -943,10 +998,6 @@ def test_create_ingress_rule(
     assert response.description == "description_value"
 
 
-def test_create_ingress_rule_from_dict():
-    test_create_ingress_rule(request_type=dict)
-
-
 def test_create_ingress_rule_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -1064,9 +1115,8 @@ async def test_create_ingress_rule_field_headers_async():
     assert ("x-goog-request-params", "parent=parent/value",) in kw["metadata"]
 
 
-def test_get_ingress_rule(
-    transport: str = "grpc", request_type=appengine.GetIngressRuleRequest
-):
+@pytest.mark.parametrize("request_type", [appengine.GetIngressRuleRequest, dict,])
+def test_get_ingress_rule(request_type, transport: str = "grpc"):
     client = FirewallClient(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -1099,10 +1149,6 @@ def test_get_ingress_rule(
     assert response.description == "description_value"
 
 
-def test_get_ingress_rule_from_dict():
-    test_get_ingress_rule(request_type=dict)
-
-
 def test_get_ingress_rule_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -1212,9 +1258,8 @@ async def test_get_ingress_rule_field_headers_async():
     assert ("x-goog-request-params", "name=name/value",) in kw["metadata"]
 
 
-def test_update_ingress_rule(
-    transport: str = "grpc", request_type=appengine.UpdateIngressRuleRequest
-):
+@pytest.mark.parametrize("request_type", [appengine.UpdateIngressRuleRequest, dict,])
+def test_update_ingress_rule(request_type, transport: str = "grpc"):
     client = FirewallClient(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -1249,10 +1294,6 @@ def test_update_ingress_rule(
     assert response.description == "description_value"
 
 
-def test_update_ingress_rule_from_dict():
-    test_update_ingress_rule(request_type=dict)
-
-
 def test_update_ingress_rule_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -1370,9 +1411,8 @@ async def test_update_ingress_rule_field_headers_async():
     assert ("x-goog-request-params", "name=name/value",) in kw["metadata"]
 
 
-def test_delete_ingress_rule(
-    transport: str = "grpc", request_type=appengine.DeleteIngressRuleRequest
-):
+@pytest.mark.parametrize("request_type", [appengine.DeleteIngressRuleRequest, dict,])
+def test_delete_ingress_rule(request_type, transport: str = "grpc"):
     client = FirewallClient(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -1398,10 +1438,6 @@ def test_delete_ingress_rule(
     assert response is None
 
 
-def test_delete_ingress_rule_from_dict():
-    test_delete_ingress_rule(request_type=dict)
-
-
 def test_delete_ingress_rule_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -1526,6 +1562,23 @@ def test_credentials_transport_error():
             transport=transport,
         )
 
+    # It is an error to provide an api_key and a transport instance.
+    transport = transports.FirewallGrpcTransport(
+        credentials=ga_credentials.AnonymousCredentials(),
+    )
+    options = client_options.ClientOptions()
+    options.api_key = "api_key"
+    with pytest.raises(ValueError):
+        client = FirewallClient(client_options=options, transport=transport,)
+
+    # It is an error to provide an api_key and a credential.
+    options = mock.Mock()
+    options.api_key = "api_key"
+    with pytest.raises(ValueError):
+        client = FirewallClient(
+            client_options=options, credentials=ga_credentials.AnonymousCredentials()
+        )
+
     # It is an error to provide scopes and a transport instance.
     transport = transports.FirewallGrpcTransport(
         credentials=ga_credentials.AnonymousCredentials(),
@@ -1611,8 +1664,10 @@ def test_firewall_base_transport():
         with pytest.raises(NotImplementedError):
             getattr(transport, method)(request=object())
 
+    with pytest.raises(NotImplementedError):
+        transport.close()
+
 
-@requires_google_auth_gte_1_25_0
 def test_firewall_base_transport_with_credentials_file():
     # Instantiate the base transport with a credentials file
     with mock.patch.object(
@@ -1637,30 +1692,6 @@ def test_firewall_base_transport_with_credentials_file():
         )
 
 
-@requires_google_auth_lt_1_25_0
-def test_firewall_base_transport_with_credentials_file_old_google_auth():
-    # Instantiate the base transport with a credentials file
-    with mock.patch.object(
-        google.auth, "load_credentials_from_file", autospec=True
-    ) as load_creds, mock.patch(
-        "google.cloud.appengine_admin_v1.services.firewall.transports.FirewallTransport._prep_wrapped_messages"
-    ) as Transport:
-        Transport.return_value = None
-        load_creds.return_value = (ga_credentials.AnonymousCredentials(), None)
-        transport = transports.FirewallTransport(
-            credentials_file="credentials.json", quota_project_id="octopus",
-        )
-        load_creds.assert_called_once_with(
-            "credentials.json",
-            scopes=(
-                "https://www.googleapis.com/auth/appengine.admin",
-                "https://www.googleapis.com/auth/cloud-platform",
-                "https://www.googleapis.com/auth/cloud-platform.read-only",
-            ),
-            quota_project_id="octopus",
-        )
-
-
 def test_firewall_base_transport_with_adc():
     # Test the default credentials are used if credentials and credentials_file are None.
     with mock.patch.object(google.auth, "default", autospec=True) as adc, mock.patch(
@@ -1672,7 +1703,6 @@ def test_firewall_base_transport_with_adc():
         adc.assert_called_once()
 
 
-@requires_google_auth_gte_1_25_0
 def test_firewall_auth_adc():
     # If no credentials are provided, we should use ADC credentials.
     with mock.patch.object(google.auth, "default", autospec=True) as adc:
@@ -1689,27 +1719,10 @@ def test_firewall_auth_adc():
         )
 
 
-@requires_google_auth_lt_1_25_0
-def test_firewall_auth_adc_old_google_auth():
-    # If no credentials are provided, we should use ADC credentials.
-    with mock.patch.object(google.auth, "default", autospec=True) as adc:
-        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
-        FirewallClient()
-        adc.assert_called_once_with(
-            scopes=(
-                "https://www.googleapis.com/auth/appengine.admin",
-                "https://www.googleapis.com/auth/cloud-platform",
-                "https://www.googleapis.com/auth/cloud-platform.read-only",
-            ),
-            quota_project_id=None,
-        )
-
-
 @pytest.mark.parametrize(
     "transport_class",
     [transports.FirewallGrpcTransport, transports.FirewallGrpcAsyncIOTransport,],
 )
-@requires_google_auth_gte_1_25_0
 def test_firewall_transport_auth_adc(transport_class):
     # If credentials and host are not provided, the transport class should use
     # ADC credentials.
@@ -1727,27 +1740,6 @@ def test_firewall_transport_auth_adc(transport_class):
         )
 
 
-@pytest.mark.parametrize(
-    "transport_class",
-    [transports.FirewallGrpcTransport, transports.FirewallGrpcAsyncIOTransport,],
-)
-@requires_google_auth_lt_1_25_0
-def test_firewall_transport_auth_adc_old_google_auth(transport_class):
-    # If credentials and host are not provided, the transport class should use
-    # ADC credentials.
-    with mock.patch.object(google.auth, "default", autospec=True) as adc:
-        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
-        transport_class(quota_project_id="octopus")
-        adc.assert_called_once_with(
-            scopes=(
-                "https://www.googleapis.com/auth/appengine.admin",
-                "https://www.googleapis.com/auth/cloud-platform",
-                "https://www.googleapis.com/auth/cloud-platform.read-only",
-            ),
-            quota_project_id="octopus",
-        )
-
-
 @pytest.mark.parametrize(
     "transport_class,grpc_helpers",
     [
@@ -2061,7 +2053,7 @@ def test_parse_common_location_path():
     assert expected == actual
 
 
-def test_client_withDEFAULT_CLIENT_INFO():
+def test_client_with_default_client_info():
     client_info = gapic_v1.client_info.ClientInfo()
 
     with mock.patch.object(
@@ -2080,3 +2072,79 @@ def test_client_withDEFAULT_CLIENT_INFO():
             credentials=ga_credentials.AnonymousCredentials(), client_info=client_info,
         )
         prep.assert_called_once_with(client_info)
+
+
+@pytest.mark.asyncio
+async def test_transport_close_async():
+    client = FirewallAsyncClient(
+        credentials=ga_credentials.AnonymousCredentials(), transport="grpc_asyncio",
+    )
+    with mock.patch.object(
+        type(getattr(client.transport, "grpc_channel")), "close"
+    ) as close:
+        async with client:
+            close.assert_not_called()
+        close.assert_called_once()
+
+
+def test_transport_close():
+    transports = {
+        "grpc": "_grpc_channel",
+    }
+
+    for transport, close_name in transports.items():
+        client = FirewallClient(
+            credentials=ga_credentials.AnonymousCredentials(), transport=transport
+        )
+        with mock.patch.object(
+            type(getattr(client.transport, close_name)), "close"
+        ) as close:
+            with client:
+                close.assert_not_called()
+            close.assert_called_once()
+
+
+def test_client_ctx():
+    transports = [
+        "grpc",
+    ]
+    for transport in transports:
+        client = FirewallClient(
+            credentials=ga_credentials.AnonymousCredentials(), transport=transport
+        )
+        # Test client calls underlying transport.
+        with mock.patch.object(type(client.transport), "close") as close:
+            close.assert_not_called()
+            with client:
+                pass
+            close.assert_called()
+
+
+@pytest.mark.parametrize(
+    "client_class,transport_class",
+    [
+        (FirewallClient, transports.FirewallGrpcTransport),
+        (FirewallAsyncClient, transports.FirewallGrpcAsyncIOTransport),
+    ],
+)
+def test_api_key_credentials(client_class, transport_class):
+    with mock.patch.object(
+        google.auth._default, "get_api_key_credentials", create=True
+    ) as get_api_key_credentials:
+        mock_cred = mock.Mock()
+        get_api_key_credentials.return_value = mock_cred
+        options = client_options.ClientOptions()
+        options.api_key = "api_key"
+        with mock.patch.object(transport_class, "__init__") as patched:
+            patched.return_value = None
+            client = client_class(client_options=options)
+            patched.assert_called_once_with(
+                credentials=mock_cred,
+                credentials_file=None,
+                host=client.DEFAULT_ENDPOINT,
+                scopes=None,
+                client_cert_source_for_mtls=None,
+                quota_project_id=None,
+                client_info=transports.base.DEFAULT_CLIENT_INFO,
+                always_use_jwt_access=True,
+            )
